use axum::extract::rejection::JsonRejection;
use axum::{
    async_trait,
    extract::FromRequest,
    http::{header::CONTENT_TYPE, Request},
    response::{IntoResponse, Response},
    RequestExt,
};
use axum_extra::extract::FormRejection;
use axum_macros::FromRequest;
use serde::Serialize;

use crate::error::{ErrorString, ErrorStringResult};

// convert a serializable struct into a json response for ErrorStringResult
pub fn json_response<T>(data: &T) -> ErrorStringResult
where
    T: Serialize,
{
    Ok(axum::Json(data).into_response())
}

// In order to have the Json and Form extracted into our ErrorString json error
// we need to implement our own extractors based on the real ones...
// XXX Why can't we map error directly in map_err below?!
#[derive(FromRequest)]
#[from_request(via(axum::Json), rejection(JsonErrorRejection))]
struct Json<T>(T);

#[derive(FromRequest)]
#[from_request(via(axum_extra::extract::Form), rejection(JsonErrorRejection))]
struct Form<T>(T);

struct JsonErrorRejection(ErrorString);

impl From<FormRejection> for JsonErrorRejection {
    fn from(rejection: FormRejection) -> Self {
        let body = match rejection {
            FormRejection::RawFormRejection(inner) => inner.body_text(),
            FormRejection::FailedToDeserializeForm(inner) => inner.to_string(),
            _ => rejection.to_string(), // should never happen, but enum is marked non_exhaustive
        };
        let response = ErrorString { error: body };

        JsonErrorRejection(response)
    }
}

impl From<JsonRejection> for JsonErrorRejection {
    fn from(rejection: JsonRejection) -> Self {
        let response = ErrorString {
            error: rejection.body_text(),
        };

        JsonErrorRejection(response)
    }
}

impl IntoResponse for JsonErrorRejection {
    fn into_response(self) -> Response {
        self.0.into_response()
    }
}

pub struct JsonOrFormOption<T>(pub Option<T>);

#[async_trait]
impl<S, B, T> FromRequest<S, B> for JsonOrFormOption<T>
where
    Json<T>: FromRequest<(), B>,
    Form<T>: FromRequest<(), B>,
    T: 'static,
    S: Send + Sync,
    B: Send + 'static,
{
    type Rejection = Response;

    async fn from_request(req: Request<B>, _state: &S) -> Result<Self, Self::Rejection> {
        let Some(content_type) = req
            .headers()
            .get(CONTENT_TYPE)
            .and_then(|value| value.to_str().ok())
            .map(|value| value.to_lowercase())
        else {
            return Ok(JsonOrFormOption(None));
        };
        if content_type.starts_with("application/json") {
            let Json(payload) = req
                .extract::<Json<T>, _>()
                .await
                .map_err(|err| err.into_response())?;
            Ok(JsonOrFormOption(Some(payload)))
        } else if content_type.starts_with("application/x-www-form-urlencoded") {
            let Form(payload) = req
                .extract::<Form<T>, _>()
                .await
                .map_err(|err| err.into_response())?;
            Ok(JsonOrFormOption(Some(payload)))
        } else {
            Ok(JsonOrFormOption(None))
        }
    }
}

pub struct JsonOrForm<T>(pub T);

#[async_trait]
impl<S, B, T> FromRequest<S, B> for JsonOrForm<T>
where
    Json<T>: FromRequest<(), B>,
    Form<T>: FromRequest<(), B>,
    T: 'static,
    S: Send + Sync,
    B: Send + 'static,
{
    type Rejection = Response;

    async fn from_request(req: Request<B>, state: &S) -> Result<Self, Self::Rejection> {
        match JsonOrFormOption::from_request(req, state).await? {
           JsonOrFormOption(None) => {
               Err(ErrorString {
                   error: "`content-type` must be set (application/json or application/x-www-form-urlencoded)".to_string()
               }.into_response())?
           },
            JsonOrFormOption(Some(payload)) => Ok(JsonOrForm(payload))
        }
    }
}
