// SPDX-License-Identifier: MIT

use anyhow::Context;
use askama::Template;
use axum::{
    extract::Form,
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Router,
};
use serde::Deserialize;
use std::collections::HashMap;

use crate::common;
use crate::common::{
    html_template::HtmlTemplate,
    networkmanager::{nmconvec, NMCon},
    CheckAuth,
};
use crate::error::PageResult;

#[derive(Deserialize)]
struct DhcpParam {
    interface: String,
    start_addr: String,
    end_addr: String,
    lease_time: String,
}

#[derive(Deserialize)]
struct DhcpDelParam {
    interface: String,
}

struct DhcpCandidate {
    interface: String,
    ip: String,
}

#[derive(Template)]
#[template(path = "../src/dhcp/templates/dhcp.html")]
struct DhcpTemplate {
    candidates: Vec<DhcpCandidate>,
    dhcp_infos: Vec<HashMap<&'static str, String>>,
}

pub trait DhcpRoutes {
    fn dhcp_routes(self) -> Self;
}

impl DhcpRoutes for Router {
    fn dhcp_routes(self) -> Self {
        self.route("/dhcp", get(dhcp))
            .route("/dhcp_setup", post(dhcp_setup))
            .route("/dhcp_delete", post(dhcp_delete))
    }
}

async fn dhcp(_loggedin: CheckAuth) -> PageResult {
    let cons = nmconvec().await?;
    let mut candidates = Vec::<DhcpCandidate>::new();
    let mut dhcp_infos = Vec::<HashMap<&'static str, String>>::new();
    for con in cons {
        // cons values don't have enough info, will add a helper to enrich later
        // but for now fetch manually...
        let Ok(con) = NMCon::from_id(&con.uuid).await else {
            continue;
        };
        // ignore interfaces not in manual mode
        if con
            .prop("ipv4.method")
            .map(|v| v != "manual")
            .unwrap_or(true)
        {
            continue;
        }
        if con.device.is_none() {
            continue;
        }
        let interface = con
            .device
            .as_ref()
            .context("no interface after checking none")?
            .as_str();
        // ignore podman interfaces
        if interface.starts_with("podman") {
            continue;
        }
        let dhcp_info = common::dhcp_info(interface).await?;
        if dhcp_info.is_empty() {
            // candidate
            if let Some(ip) = con.prop("IP4.ADDRESS[1]") {
                candidates.push(DhcpCandidate {
                    interface: interface.to_string(),
                    ip: ip.to_string(),
                });
            }
        } else {
            dhcp_infos.push(dhcp_info);
        }
    }

    candidates.sort_unstable_by(|a, b| a.interface.cmp(&b.interface));
    dhcp_infos.sort_unstable_by(|a, b| a["interface"].cmp(&b["interface"]));

    let template = DhcpTemplate {
        candidates,
        dhcp_infos,
    };
    Ok(HtmlTemplate::new("DHCPサーバー設定", template).into_response())
}

async fn dhcp_setup(_loggedin: CheckAuth, Form(dhcp_param): Form<DhcpParam>) -> PageResult {
    let args = &[
        "dhcp_setup.sh",
        &dhcp_param.start_addr,
        &dhcp_param.end_addr,
        &dhcp_param.lease_time,
        &dhcp_param.interface,
    ];
    common::exec_command(args).await?;
    Ok(Redirect::to("/dhcp").into_response())
}

async fn dhcp_delete(_loggedin: CheckAuth, Form(dhcp_del_param): Form<DhcpDelParam>) -> PageResult {
    let args = &["dhcp_delete.sh", &dhcp_del_param.interface];
    common::exec_command(args).await?;
    Ok(Redirect::to("/dhcp").into_response())
}
