// SPDX-License-Identifier: MIT

use anyhow::Result;
use axum::error_handling::HandleErrorLayer;
use axum::{http::StatusCode, BoxError, Router};
use tower::ServiceBuilder;
use tower_http::services::ServeDir;
use tower_sessions::{Expiry, SessionManagerLayer};

// common subdirs
mod args;
mod common;
mod error;
mod ip_filter;
mod server;
mod session_store;

// pages
mod connection;
mod container;
mod dhcp;
mod login;
mod nat;
mod nwstatbar;
mod power_management;
mod settings;
mod status;
mod swu;
mod top;
mod vpn;
mod wlan;
mod wwan;
use connection::ConnectionRoutes;
use container::ContainerRoutes;
use dhcp::DhcpRoutes;
use login::LoginRoutes;
use nat::NatRoutes;
use nwstatbar::NWStatBarRoutes;
use power_management::PowerManagementRoutes;
use settings::SettingsRoutes;
use status::StatusRoutes;
use swu::SwuRoutes;
use top::TopRoutes;
use vpn::VpnRoutes;
use wlan::WlanRoutes;
use wwan::WwanRoutes;

#[cfg(debug_assertions)]
const STATIC_RESOURCE_DIR: &str = "static";

#[cfg(not(debug_assertions))]
const STATIC_RESOURCE_DIR: &str = "/usr/share/abos-web/static";

async fn fallback_404() -> (StatusCode, &'static str) {
    (StatusCode::NOT_FOUND, "Page not found")
}

#[tokio::main]
async fn main() -> Result<()> {
    let session_store = session_store::MemoryStore::new(std::time::Duration::from_secs(12 * 3600));
    let session_service = ServiceBuilder::new()
        .layer(HandleErrorLayer::new(|_: BoxError| async {
            StatusCode::BAD_REQUEST
        }))
        .layer(
            SessionManagerLayer::new(session_store)
                .with_secure(false)
                .with_expiry(Expiry::OnInactivity(time::Duration::hours(12))),
        );

    common::Config::load().await?;
    common::password_throttle_init();
    args::args_init();

    let app = Router::new()
        .connection_routes()
        .container_routes()
        .dhcp_routes()
        .login_routes()
        .nat_routes()
        .nwstatbar_routes()
        .power_management_routes()
        .settings_routes()
        .status_routes()
        .swu_routes()
        .top_routes()
        .vpn_routes()
        .wlan_routes()
        .wwan_routes()
        .nest_service("/static", ServeDir::new(STATIC_RESOURCE_DIR))
        .layer(session_service)
        .route_layer(axum::middleware::from_fn(ip_filter::ipaddr_filter))
        .fallback(fallback_404);

    server::run_server(app).await;
    Ok(())
}
