// SPDX-License-Identifier: MIT

#[cfg(feature = "restapi")]
use anyhow::Context;

use askama::Template;
use axum::{
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Router,
};
// need axum_extra's form to handle permissions list
use axum_extra::extract::Form;
use serde::Deserialize;

use crate::common::{check_password, exec_command_stdin, html_template::HtmlTemplate, CheckAuth};
use crate::error::PageResult;

#[cfg(feature = "restapi")]
use crate::common::{restapi, RestApiPermission, RestApiToken};

#[cfg(feature = "restapi")]
mod restapi_rest_tokens;

#[cfg(feature = "restapi")]
#[derive(Template)]
#[template(path = "../src/settings/templates/settings_restapi.html")]
struct SettingsRestapiTemplate {
    tokens: Vec<RestApiToken>,
}

#[derive(Template)]
#[template(path = "../src/settings/templates/settings.html")]
struct SettingsTemplate {
    #[cfg(feature = "restapi")]
    restapi: SettingsRestapiTemplate,
    #[cfg(not(feature = "restapi"))]
    restapi: &'static str,
}

#[cfg(feature = "restapi")]
#[derive(Template)]
#[template(path = "../src/settings/templates/settings_restapi_setup.html")]
struct SettingsRestapiSetupTemplate {
    token_id: Option<String>,
    permissions: Vec<RestApiPermission>,
}

pub trait SettingsRoutes {
    fn settings_routes(self) -> Self;
}

impl SettingsRoutes for Router {
    fn settings_routes(self) -> Self {
        let routes = self;

        #[cfg(feature = "restapi")]
        let routes = restapi_rest_tokens::routes(routes)
            .route("/settings_rest_token_delete", post(settings_token_delete))
            .route("/settings_rest_token_setup", get(settings_token_setup))
            .route("/settings_rest_token_save", post(settings_token_save));

        routes
            .route("/settings", get(settings))
            .route("/settings_password", post(settings_password))
    }
}

async fn settings(_loggedin: CheckAuth) -> PageResult {
    #[cfg(feature = "restapi")]
    let tokens = restapi::list_tokens().await?;
    let template = SettingsTemplate {
        #[cfg(feature = "restapi")]
        restapi: SettingsRestapiTemplate { tokens },
        #[cfg(not(feature = "restapi"))]
        restapi: "",
    };
    Ok(HtmlTemplate::new("設定管理", template).into_response())
}

#[derive(Deserialize)]
struct ChangePasswordParam {
    current_password: String,
    password: String,
    password_confirm: String,
}

async fn settings_password(
    _loggedin: CheckAuth,
    Form(param): Form<ChangePasswordParam>,
) -> PageResult {
    check_password(&param.current_password).await?;
    if param.password != param.password_confirm {
        Err("登録したパスワードが一致しません。")?;
    }
    exec_command_stdin(&["register_password.sh"], &param.password).await?;
    Ok(Redirect::to("/settings").into_response())
}

#[cfg(feature = "restapi")]
#[derive(Deserialize)]
struct TokenIdParam {
    token_id: String,
}

#[cfg(feature = "restapi")]
async fn settings_token_delete(
    _loggedin: CheckAuth,
    Form(param): Form<TokenIdParam>,
) -> PageResult {
    restapi::delete_token(&param.token_id).await?;
    Ok(Redirect::to("/settings").into_response())
}

#[cfg(feature = "restapi")]
#[derive(Deserialize)]
struct TokenIdOrNewParam {
    token_id: Option<String>,
    new_token: Option<bool>,
}

#[cfg(feature = "restapi")]
async fn settings_token_setup(
    _loggedin: CheckAuth,
    Form(param): Form<TokenIdOrNewParam>,
) -> PageResult {
    let (token_id, permissions) = if param.new_token.unwrap_or(false) {
        (None, vec![RestApiPermission::Admin])
    } else {
        let token = restapi::get_token(&param.token_id.context("No token given")?).await?;
        (Some(token.token_id), token.permissions)
    };
    let template = SettingsRestapiSetupTemplate {
        token_id,
        permissions,
    };
    Ok(HtmlTemplate::new("Rest API トークン編集", template).into_response())
}

#[cfg(feature = "restapi")]
#[derive(Deserialize)]
struct TokenParam {
    token_id: Option<String>,
    permissions: Vec<RestApiPermission>,
}

#[cfg(feature = "restapi")]
async fn settings_token_save(_loggedin: CheckAuth, Form(param): Form<TokenParam>) -> PageResult {
    match param.token_id {
        None => {
            restapi::create_token(param.permissions).await?;
        }
        Some(token_id) => {
            let mut token = restapi::get_token(&token_id).await.unwrap_or(RestApiToken {
                token_id,
                permissions: vec![],
            });
            token.permissions = param.permissions;
            restapi::write_token(&token).await?;
        }
    }
    Ok(Redirect::to("/settings").into_response())
}
